@@ -70,3 +70,30 @@ construct_path <- function(...)
 {
     sub("/$", "", file.path(..., fsep="/"))
 }
+
+
+# combine several pages of objects into a single list
+get_paged_list <- function(lst, token, next_link_name="nextLink", value_name="value")
+{
+    res <- lst[[value_name]]
+    while(!is_empty(lst[[next_link_name]]))
+    {
+        lst <- call_azure_url(token, lst[[next_link_name]])
+        res <- c(res, lst[[value_name]])
+    }
+    res    
+}
+
+
+# TRUE if delete confirmed, FALSE otherwise
+delete_confirmed <- function(confirm, name, type, quote_name=TRUE)
+{
+    if(!interactive() || !confirm)
+        return(TRUE)
+    
+    msg <- if(quote_name)
+        sprintf("Do you really want to delete the %s '%s'? (y/N) ", type, name)
+    else sprintf("Do you really want to delete the %s %s? (y/N) ", type, name)
+    yn <- readline(msg)
+    return(tolower(substr(yn, 1, 1)) == "y")
+}
