@@ -9,18 +9,17 @@ if(tenant == "" || app == "" || password == "" || subscription == "")
     skip("Authentication tests skipped: ARM credentials not set")
 
 
+suppressWarnings(file.remove(file.path(AzureR_dir(), "arm_logins.json")))
+
 test_that("ARM authentication works",
 {
     az <- az_rm$new(tenant=tenant, app=app, password=password)
     expect_is(az, "az_rm")
     expect_true(is_azure_token(az$token))
 
-    creds <- tempfile(fileext=".json")
-    writeLines(jsonlite::toJSON(list(tenant=tenant, app=app, password=password)), creds)
-                        
-    az2 <- az_rm$new(config_file=creds)
+    tok <- get_azure_token("https://management.azure.com/", tenant, app, password)
+    az2 <- az_rm$new(token=tok)
     expect_is(az2, "az_rm")
-    expect_true(is_azure_token(az2$token))
 })
 
 test_that("Login interface works",
@@ -35,9 +34,14 @@ test_that("Login interface works",
     writeLines(jsonlite::toJSON(list(tenant=tenant, app=app, password=password)), creds)
 
     az4 <- create_azure_login(config_file=creds)
+    expect_identical(normalize_tenant(tenant), az4$tenant)
     expect_is(az4, "az_rm")
 
     az5 <- get_azure_login(tenant)
     expect_is(az5, "az_rm")
+
+    tok <- get_azure_token("https://management.azure.com/", tenant, app, password)
+    az6 <- create_azure_login(token=tok)
+    expect_is(az6, "az_rm")
 })
 
